'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const BASE = '가'.charCodeAt(0); // 한글 코드 시작: 44032
const INITIALS = ['ㄱ', 'ㄲ', 'ㄴ', 'ㄷ', 'ㄸ', 'ㄹ', 'ㅁ', 'ㅂ', 'ㅃ', 'ㅅ', 'ㅆ', 'ㅇ', 'ㅈ', 'ㅉ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ'];
const MEDIALS = ['ㅏ', 'ㅐ', 'ㅑ', 'ㅒ', 'ㅓ', 'ㅔ', 'ㅕ', 'ㅖ', 'ㅗ', 'ㅘ', 'ㅙ', 'ㅚ', 'ㅛ', 'ㅜ', 'ㅝ', 'ㅞ', 'ㅟ', 'ㅠ', 'ㅡ', 'ㅢ', 'ㅣ'];
const FINALES = ['', 'ㄱ', 'ㄲ', 'ㄳ', 'ㄴ', 'ㄵ', 'ㄶ', 'ㄷ', 'ㄹ', 'ㄺ', 'ㄻ', 'ㄼ', 'ㄽ', 'ㄾ', 'ㄿ', 'ㅀ', 'ㅁ', 'ㅂ', 'ㅄ', 'ㅅ', 'ㅆ', 'ㅇ', 'ㅈ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ'];
const MIXED = {
    ㄲ: ['ㄱ', 'ㄱ'],
    ㄳ: ['ㄱ', 'ㅅ'],
    ㄵ: ['ㄴ', 'ㅈ'],
    ㄶ: ['ㄴ', 'ㅎ'],
    ㄺ: ['ㄹ', 'ㄱ'],
    ㄻ: ['ㄹ', 'ㅁ'],
    ㄼ: ['ㄹ', 'ㅂ'],
    ㄽ: ['ㄹ', 'ㅅ'],
    ㄾ: ['ㄹ', 'ㅌ'],
    ㄿ: ['ㄹ', 'ㅍ'],
    ㅀ: ['ㄹ', 'ㅎ'],
    ㅄ: ['ㅂ', 'ㅅ'],
    ㅆ: ['ㅅ', 'ㅅ'],
    ㅘ: ['ㅗ', 'ㅏ'],
    ㅙ: ['ㅗ', 'ㅐ'],
    ㅚ: ['ㅗ', 'ㅣ'],
    ㅝ: ['ㅜ', 'ㅓ'],
    ㅞ: ['ㅜ', 'ㅔ'],
    ㅟ: ['ㅜ', 'ㅣ'],
    ㅢ: ['ㅡ', 'ㅣ'],
};
const MEDIAL_RANGE = {
    ㅗ: ['ㅗ', 'ㅚ'],
    ㅜ: ['ㅜ', 'ㅟ'],
    ㅡ: ['ㅡ', 'ㅢ'],
};
const PRESENT_ON_KEYBOARD = [
    'ㄱ', 'ㄲ', 'ㄴ', 'ㄷ', 'ㄸ', 'ㄹ', 'ㅁ', 'ㅂ', 'ㅃ', 'ㅅ', 'ㅆ', 'ㅇ', 'ㅈ', 'ㅉ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ',
    'ㅏ', 'ㅐ', 'ㅑ', 'ㅒ', 'ㅓ', 'ㅔ', 'ㅕ', 'ㅖ', 'ㅗ', 'ㅛ', 'ㅜ', 'ㅠ', 'ㅡ', 'ㅣ',
];
const KEYS = [
    ['ㄱ', 'r'],
    ['ㄲ', 'R'],
    ['ㄴ', 's'],
    ['ㄷ', 'e'],
    ['ㄸ', 'E'],
    ['ㄹ', 'f'],
    ['ㅁ', 'a'],
    ['ㅂ', 'q'],
    ['ㅃ', 'Q'],
    ['ㅅ', 't'],
    ['ㅆ', 'T'],
    ['ㅇ', 'd'],
    ['ㅈ', 'w'],
    ['ㅉ', 'W'],
    ['ㅊ', 'c'],
    ['ㅋ', 'z'],
    ['ㅌ', 'x'],
    ['ㅍ', 'v'],
    ['ㅎ', 'g'],
    ['ㅏ', 'k'],
    ['ㅐ', 'o'],
    ['ㅑ', 'i'],
    ['ㅒ', 'O'],
    ['ㅓ', 'j'],
    ['ㅔ', 'p'],
    ['ㅕ', 'u'],
    ['ㅖ', 'P'],
    ['ㅗ', 'h'],
    ['ㅛ', 'y'],
    ['ㅜ', 'n'],
    ['ㅠ', 'b'],
    ['ㅡ', 'm'],
    ['ㅣ', 'l'],
];

function getPhonemes(char) {
    let initial = '';
    let medial = '';
    let finale = '';
    let initialOffset = -1;
    let medialOffset = -1;
    let finaleOffset = -1;
    if (char.match(/[ㄱ-ㅎ]/)) {
        initial = char;
        initialOffset = INITIALS.join('').search(char);
    }
    else if (char.match(/[가-힣]/)) {
        const tmp = char.charCodeAt(0) - BASE;
        finaleOffset = tmp % FINALES.length;
        medialOffset = ((tmp - finaleOffset) / FINALES.length) % MEDIALS.length;
        initialOffset = ((tmp - finaleOffset) / FINALES.length - medialOffset) / MEDIALS.length;
        initial = INITIALS[initialOffset];
        medial = MEDIALS[medialOffset];
        finale = FINALES[finaleOffset];
    }
    return { initial, medial, finale, initialOffset, medialOffset, finaleOffset };
}

const postPositions = [
    ['은', '는'],
    ['이', '가'],
    ['을', '를'],
    ['과', '와'],
].reduce((accum, [p1, p2]) => ([
    ...accum,
    [RegExp(`([가-힣]['" ]{0,1})${p1}\\(${p2}\\)(\\s+)`, 'g'), p1, p2],
    [RegExp(`([가-힣]['" ]{0,1})${p2}\\(${p1}\\)(\\s+)`, 'g'), p1, p2],
]), []);
function correctPostpositions(text) {
    return postPositions.reduce((prev, [pattern, p1, p2]) => prev.replace(pattern, (whole, a1, a2) => `${a1.replace(/\s+$/, '')}${getPhonemes(a1).finale ? p1 : p2}${a2}`), text);
}

function explode(text, { grouped = false } = {}) {
    const accum = [];
    text.split('').forEach((char) => {
        const { initial, medial, finale, initialOffset, medialOffset, finaleOffset } = getPhonemes(char);
        accum.push((initialOffset !== -1 || medialOffset !== -1 || finaleOffset !== -1
            ? [initial, MIXED[medial] && PRESENT_ON_KEYBOARD.indexOf(medial) === -1 ? MIXED[medial] : medial, MIXED[finale] && PRESENT_ON_KEYBOARD.indexOf(finale) === -1 ? MIXED[finale] : finale].filter(Boolean)
            : [char]).flat());
    });
    return grouped ? accum : accum.flat();
}

const complexDict = Object.entries(MIXED).reduce((accum, [k, v]) => (Object.assign(Object.assign({}, accum), { [v.join('')]: k })), {});
const isNotUndefined = (e) => typeof e !== 'undefined';
function assemble(arr) {
    const startIndex = arr.findIndex((e) => MEDIALS.indexOf(e) !== -1);
    const endIndex = startIndex !== -1 && MEDIALS.indexOf(arr[startIndex + 1]) !== -1 ? startIndex + 1 : startIndex;
    let initial = arr.slice(0, startIndex).join('');
    let medial = arr.slice(startIndex, endIndex + 1).join('');
    let finale = arr.slice(endIndex + 1).join('');
    const initialOffset = INITIALS.indexOf(complexDict[initial] || initial);
    const medialOffset = MEDIALS.indexOf(complexDict[medial] || medial);
    const finaleOffset = FINALES.indexOf(complexDict[finale] || finale);
    if (initialOffset !== -1 && medialOffset !== -1) {
        return String.fromCharCode(BASE + initialOffset * (MEDIALS.length * FINALES.length) + medialOffset * FINALES.length + finaleOffset);
    }
    return arr.join('');
}
function implode(input) {
    const chars = [];
    // 인접한 모음을 하나의 복합 모음으로 합친다.
    (typeof input === 'string' ? input.split('') : input).forEach((e, i, arr) => {
        if (typeof e === 'string' && chars.length > 0 && MEDIALS.indexOf(arr[i - 1]) !== -1 && MEDIALS.indexOf(e) !== -1 && complexDict[`${arr[i - 1]}${e}`]) {
            chars[chars.length - 1] = complexDict[`${arr[i - 1]}${e}`];
        }
        else {
            chars.push(e);
        }
    });
    let cursor = { medial: null, finales: [] };
    let items = [cursor];
    // 모음으로 시작하는 그룹들을 만든다. (grouped로 넘어온 항목들은 유지)
    chars.forEach((e) => {
        if (Array.isArray(e)) {
            cursor = { medial: null, finales: [] };
            items.push({ grouped: e, finales: [] });
            items.push(cursor);
        }
        else if (MEDIALS.indexOf(e) !== -1) {
            cursor = { medial: e, finales: [] };
            items.push(cursor);
        }
        else {
            cursor.finales.push(e);
        }
    });
    // 각 그룹을 순회하면서 복합자음을 정리하고, 앞 그룹에서 종성으로 사용하고 남은 자음들을 초성으로 가져온다.
    items.forEach((curr, i, arr) => {
        if (i > 0) {
            const prev = arr[i - 1];
            if (!prev.medial || prev.finales.length === 1) {
                curr.initials = prev.finales;
                prev.finales = [];
            }
            else {
                const [finale, ...initials] = prev.finales;
                curr.initials = initials;
                prev.finales = finale ? [finale] : [];
            }
            if (curr.finales.length > 2 || (i === items.length - 1 && curr.finales.length > 1)) {
                const [a, b, ...rest] = curr.finales;
                if (complexDict[`${a}${b}`]) {
                    curr.finales = [complexDict[`${a}${b}`], ...rest];
                }
            }
        }
    });
    // 각 글자에 해당하는 블록 단위로 나눈 후 조합한다.
    const groups = [];
    items.forEach(({ initials = [], medial, finales, grouped }) => {
        if (grouped) {
            groups.push(grouped);
        }
        else {
            const pre = initials.slice();
            const initial = pre.pop();
            let [finale, ...post] = finales;
            if (FINALES.indexOf(finale) === -1) {
                post = [finale, ...post];
                finale = '';
            }
            pre.filter(isNotUndefined).forEach((e) => groups.push([e]));
            groups.push([initial, medial, finale].filter(Boolean));
            post.filter(isNotUndefined).forEach((e) => groups.push([e]));
        }
    });
    return groups.map(assemble).join('');
}

const reRegExpChar = /[\\^$.*+?()[\]{}|]/g;
const reHasRegExpChar = RegExp(reRegExpChar.source);
function escapeRegExp(string) {
    return string && reHasRegExpChar.test(string) ? string.replace(reRegExpChar, '\\$&') : string || '';
}

const getInitialSearchRegExp = (initial, allowOnlyInitial = false) => {
    const initialOffset = INITIALS.indexOf(initial);
    if (initialOffset !== -1) {
        const baseCode = initialOffset * MEDIALS.length * FINALES.length + BASE;
        return `[${allowOnlyInitial ? initial : ''}${String.fromCharCode(baseCode)}-${String.fromCharCode(baseCode + MEDIALS.length * FINALES.length - 1)}]`;
    }
    return initial;
};
const FUZZY = `__${parseInt('fuzzy', 36)}__`;
const IGNORE_SPACE = `__${parseInt('ignorespace', 36)}__`;
function getRegExp(search, { initialSearch = false, startsWith = false, endsWith = false, ignoreSpace = false, ignoreCase = true, global = false, fuzzy = false } = {}) {
    let frontChars = search.split('');
    let lastChar = frontChars.slice(-1)[0];
    let lastCharPattern = '';
    const phonemes = getPhonemes(lastChar || '');
    // 마지막 글자가 한글인 경우만 수행
    if (phonemes.initialOffset !== -1) {
        frontChars = frontChars.slice(0, -1);
        const { initial, medial, finale, initialOffset, medialOffset } = phonemes;
        // 해당 초성으로 시작하는 첫번째 문자 : 가, 나, 다, ... , 하
        const baseCode = initialOffset * MEDIALS.length * FINALES.length + BASE;
        const patterns = [];
        switch (true) {
            // case 1: 종성으로 끝나는 경우 (받침이 있는 경우)
            case finale !== '': {
                // 마지막 글자
                patterns.push(lastChar);
                // 종성이 초성으로 사용 가능한 경우
                if (INITIALS.includes(finale)) {
                    patterns.push(`${String.fromCharCode(baseCode + medialOffset * FINALES.length)}${getInitialSearchRegExp(finale)}`);
                }
                // 종성이 복합 자음인 경우, 두 개의 자음으로 분리하여 각각 받침과 초성으로 사용
                if (MIXED[finale]) {
                    patterns.push(`${String.fromCharCode(baseCode + medialOffset * FINALES.length + FINALES.join('').search(MIXED[finale][0]) + 1)}${getInitialSearchRegExp(MIXED[finale][1])}`);
                }
                break;
            }
            // case 2: 중성으로 끝나는 경우 (받침이 없는 경우)
            case medial !== '': {
                let from, to;
                // 중성이 복합 모음인 경우 범위를 확장하여 적용
                if (MEDIAL_RANGE[medial]) {
                    from = baseCode + MEDIALS.join('').search(MEDIAL_RANGE[medial][0]) * FINALES.length;
                    to = baseCode + MEDIALS.join('').search(MEDIAL_RANGE[medial][1]) * FINALES.length + FINALES.length - 1;
                }
                else {
                    from = baseCode + medialOffset * FINALES.length;
                    to = from + FINALES.length - 1;
                }
                patterns.push(`[${String.fromCharCode(from)}-${String.fromCharCode(to)}]`);
                break;
            }
            // case 3: 초성만 입력된 경우
            case initial !== '': {
                patterns.push(getInitialSearchRegExp(initial, true));
                break;
            }
        }
        lastCharPattern = patterns.length > 1 ? `(${patterns.join('|')})` : patterns[0];
    }
    const glue = fuzzy ? FUZZY : ignoreSpace ? IGNORE_SPACE : '';
    const frontCharsPattern = initialSearch
        ? frontChars.map((char) => (char.search(/[ㄱ-ㅎ]/) !== -1 ? getInitialSearchRegExp(char, true) : escapeRegExp(char))).join(glue)
        : escapeRegExp(frontChars.join(glue));
    let pattern = (startsWith ? '^' : '') + frontCharsPattern + glue + lastCharPattern + (endsWith ? '$' : '');
    if (glue) {
        pattern = pattern.replace(RegExp(FUZZY, 'g'), '.*').replace(RegExp(IGNORE_SPACE, 'g'), '\\s*');
    }
    return RegExp(pattern, (global ? 'g' : '') + (ignoreCase ? 'i' : ''));
}

const EN_TO_KR = Object.values(KEYS)
    .reduce((accum, [kr, en]) => (Object.assign(Object.assign({}, accum), { [en]: kr })), {});
function engToKor(text) {
    return implode(text.split('').map(char => EN_TO_KR[char] || char).join(''));
}

const KR_TO_EN = Object.values(KEYS)
    .reduce((accum, [kr, en]) => (Object.assign(Object.assign({}, accum), { [kr]: en })), {});
function korToEng(text) {
    return text.split('').map((char) => explode(char, { grouped: false }).map((e) => KR_TO_EN[e] || e)).flat().join('');
}

exports.correctPostpositions = correctPostpositions;
exports.engToKor = engToKor;
exports.explode = explode;
exports.getPhonemes = getPhonemes;
exports.getRegExp = getRegExp;
exports.implode = implode;
exports.korToEng = korToEng;
//# sourceMappingURL=index.cjs.js.map
